<?php

/**
 * @file
 * Drupal Commerce hook implementations.
 */

/**
 * Implements hook_commerce_payment_method_info().
 */
function commerce_chase_orbital_hpf_commerce_payment_method_info() {
  $payment_methods['commerce_chase_orbital_hpf'] = array(
    'base' => 'commerce_chase_orbital_hpf',
    'file' => 'commerce_chase_orbital_hpf.commerce.inc',
    'title' => t('Orbital® Hosted Payment Form'),
    'short_title' => t('Orbital® (Hosted)'),
    'description' => t('Chase Paymentech Orbital® Hosted Payment Form'),
    'terminal' => FALSE,
    'offsite' => TRUE,
    'offsite_autoredirect' => FALSE,
  );
  return $payment_methods;
}

/**
 * Returns the default settings for the payment method.
 *
 * @return array
 *    Default settings array.
 */
function commerce_chase_orbital_hpf_default_settings() {
  return array(
    'hosted_secure_id' => '',
    'hosted_secure_api_token' => '',
    'allowed_types' => commerce_chase_orbital_hpf_allowed_types(),
    'trans_type' => 'auth_capture',
    'required' => 'minimum',
    'mode' => COMMERCE_CHASE_ORBITAL_HPF_CERTIFICATION,
    'currency_code' => commerce_default_currency(),
    'submit_text' => t('Place payment'),
    'cardonfile' => FALSE,
  );
}

/**
 * Payment method callback; return the settings form for a payment method.
 *
 * @param array $settings
 *   An array of the current settings.
 *
 * @return array
 *   A form snippet.
 */
function commerce_chase_orbital_hpf_settings_form(array $settings = array()) {
  $form = array();

  // Default to the Payflow Link payment method if none is passed in.
  if (empty($payment_method)) {
    $payment_method = commerce_payment_method_load('commerce_chase_orbital_hpf');
  }

  // Merge default settings into the stored settings array.
  $settings = (array) $settings + commerce_chase_orbital_hpf_default_settings();

  $form['hosted_secure_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Secure Account ID'),
    '#description' => t("The merchant's Hosted Payment account"),
    '#default_value' => $settings['hosted_secure_id'],
    '#required' => TRUE,
  );
  $form['hosted_secure_api_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Secure API Token'),
    '#description' => t("The merchant's Hosted Payment API token"),
    '#default_value' => $settings['hosted_secure_api_token'],
    '#required' => FALSE,
  );
  $form['allowed_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Payment option icons to show on the checkout form'),
    '#description' => t('Your payment processor and @title account settings may limit which of these payment options are actually available on the payment form.', array('@title' => $payment_method['title'])),
    '#options' => commerce_chase_orbital_hpf_allowed_types(),
    '#default_value' => array_keys($settings['allowed_types']),
  );
  $form['required'] = array(
    '#type' => 'radios',
    '#title' => t('Required card information'),
    '#options' => array(
      'minimum' => t('Card Number and Expiration Date (mandatory), Name on Card and CVV (optional)'),
      'all' => t('All fields mandatory'),
    ),
    '#default_value' => $settings['required'],
  );
  $form['mode'] = array(
    '#type' => 'radios',
    '#title' => t('Mode'),
    '#options' => array(
      COMMERCE_CHASE_ORBITAL_HPF_CERTIFICATION => t('Certification (test)'),
      COMMERCE_CHASE_ORBITAL_HPF_PRODUCTION => t('Production'),
    ),
    '#default_value' => $settings['mode'],
  );
  $form['trans_type'] = array(
    '#type' => 'radios',
    '#title' => t('Default transaction type'),
    '#options' => array(
      'auth_capture' => t('Sale - authorize and capture the funds at the time the payment is processed'),
      'auth_only' => t('Authorization - reserve funds on the card to be captured later through your Orbital virtual terminal'),
    ),
    '#default_value' => $settings['trans_type'],
  );
  if (module_exists('commerce_cardonfile')) {
    // @todo: Implement card on file
    // $form['trans_type']['#options']['store_authorize'] = t('Store and Authorize');
  }

  $form['currency_code'] = array(
    '#type' => 'select',
    '#title' => t('Default currency'),
    '#description' => t('Transactions in other currencies will be converted to this currency, so multi-currency sites must be configured to use appropriate conversion rates.') . '<br />' . t('Note: valid currencies will differ depending on your country and processor. Check before selecting a different currency.'),
    // @todo: make this a real list of options.
    '#options' => array(commerce_default_currency()),
    '#default_value' => $settings['currency_code'],
  );
  $form['submit_text'] = array(
    '#type' => 'textfield',
    '#title' => t('Submit text'),
    '#description' => t("Text on the submit button"),
    '#default_value' => $settings['submit_text'],
    '#required' => TRUE,
  );

  if (module_enable('commerce_cardonfile')) {
    $form['cardonfile'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable card on file support'),
      '#default_value' => $settings['cardonfile'],
    );
  }
  return $form;
}

/**
 * Payment method callback; generation callback for the payment submission form.
 *
 * @param $payment_method
 *   An array of the current settings.
 * @param $pane_values
 *   The current values of the pane.
 * @param $checkout_pane
 *   The checkout pane array. The checkout pane will be NULL if the payment is
 *   being added through the administration form.
 * @param $order
 *   The order object.
 * @return array
 *   A form snippet for the checkout pane.
 */
function commerce_chase_orbital_hpf_submit_form($payment_method, $pane_values, $checkout_pane, $order) {
  // No example.
}

/**
 * Payment method callback; validate callback for the payment submission form.
 *
 * @param $payment_method
 *   An array of the current settings.
 * @param $pane_form
 *   The pane form.
 * @param $pane_values
 *   The current values of the pane.
 * @param $order
 *   The order object.
 * @param $form_parents
 *   The identifier of the base element of the payment pane.
 *
 * @return bool
 */
function commerce_chase_orbital_hpf_submit_form_validate($payment_method, $pane_form, $pane_values, $order, $form_parents = array()) {
  // Return an error if the enabling action's settings haven't been configured.
  foreach (array('hosted_secure_id') as $key) {
    if (empty($payment_method['settings'][$key])) {
      drupal_set_message(t('@title is not configured for use. Please contact an administrator to resolve this issue.', array('@title' => $payment_method['title'])), 'error');
      return FALSE;
    }
  }

  return TRUE;
}

/**
 * Payment method callback; validate callback for the payment submission form.
 *
 * @param $payment_method
 *   An array of the current settings.
 * @param $pane_form
 *   The pane form.
 * @param $pane_values
 *   The current values of the pane.
 * @param $order
 *   The order object.
 * @param $charge
 *   A price structure that needs to be charged.
 */
function commerce_chase_orbital_hpf_submit_form_submit($payment_method, $pane_form, $pane_values, $order, $charge) {
  // Update the order to reference the Payflow Link payment method.
  $order->data['payment_method'] = $payment_method['instance_id'];

  // Generate a payment redirect key and Secure Token ID.
  $order->data['payment_redirect_key'] = drupal_hash_base64(time());
}

/**
 * Payment method callback; generation callback for the payment redirect form.
 *
 * @param $form
 * @param $form_state
 * @param $order
 * @param $payment_method
 *
 * @return mixed
 */
function commerce_chase_orbital_hpf_redirect_form($form, &$form_state, $order, $payment_method) {
  // Store the Payflow payment method instance in the form array.
  $form['commerce_chase_orbital_hpf_payment_method'] = array(
    '#type' => 'value',
    '#value' => $payment_method,
  );

  // Add the iframe in a markup element.
  $form['#attached']['js'][] = drupal_get_path('module', 'commerce_chase_orbital_hpf') . '/commerce-chase-orbital-hpf.js';
  $form['iframe'] = array(
    '#markup' => commerce_chase_orbital_hpf_checkout_iframe($payment_method, $order),
  );
  $form['iframe']['#suffix'] = '<div class="commerce-orbital-hpf-cancel">' . l(t('Cancel payment and go back'), 'checkout/' . $order->order_id . '/payment/back/' . $order->data['payment_redirect_key']) . '</div>';

  return $form;
}

/**
 * Payment method callback; cancellation callback for the redirected payments.
 */
function commerce_chase_orbital_hpf_redirect_form_back($order, $payment_method) {

  // Remove the payment information from the order data array.
  unset($order->data['payment_method']);

  // There was an error messaged returned.
  if ($_GET['errorCode']) {
    $error_codes = array_filter(explode('|', filter_xss($_GET['errorCode'])));
    foreach ($error_codes as $code) {
      drupal_set_message(commerce_chase_orbital_hpf_response_message($code), 'error');
    }
    if (isset($_GET['gatewayCode']) && $_GET['gatewayCode'] != 'null') {
      drupal_set_message(t('@code: @message', array(
        '@code' => filter_xss($_GET['gatewayCode']),
        '@message' => filter_xss($_GET['gatewayMessage']),
      )), 'error');
    }

    // Provide a more descriptive error message in the failed transaction and
    // the watchdog.
    $first_code = reset($error_codes);
    $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
    $order_total = $order_wrapper->commerce_order_total->value();
    $transaction = commerce_payment_transaction_new($payment_method['method_id'], $order->order_id);
    $transaction->instance_id = $payment_method['instance_id'];
    $transaction->amount = $order_total;
    $transaction->currency_code = commerce_default_currency();
    $transaction->payload[REQUEST_TIME] = $_GET;
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
    $transaction->message = commerce_chase_orbital_hpf_response_message($first_code);
    commerce_payment_transaction_save($transaction);
  }
  // The user has opted to cancel.
  else {
    drupal_set_message(t('You have canceled payment but may resume the checkout process here when you are ready.'));
  }
}

/**
 * Payment method callback; validation callback for redirected payments.
 *
 * @param $order
 *   The order object.
 * @param $payment_method
 *   The payment method array.
 * @return bool
 *   TRUE if the customer should proceed to checkout completion or FALSE to go
 *   back one step in the checkout process.
 */
function commerce_chase_orbital_hpf_redirect_form_validate($order, $payment_method) {
  // Verify transaction was sent over and can be decoded.
  if (isset($_GET['transaction']) && drupal_json_decode($_GET['transaction'])) {
    return TRUE;
  }

  // Provide a more descriptive error message in the failed transaction and
  // the watchdog.
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $order_total = $order_wrapper->commerce_order_total->value();
  $transaction = commerce_payment_transaction_new($payment_method['method_id'], $order->order_id);
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->amount = $order_total;
  $transaction->currency_code = commerce_default_currency();
  $transaction->payload[REQUEST_TIME] = $_GET;
  $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
  $transaction->message = t('Attempted to process transaction with invalid transaction payload.');
  commerce_payment_transaction_save($transaction);

  return FALSE;
}

/**
 * Payment method callback; submission callback for redirected payments.
 *
 * @param $order
 *   The order object.
 * @param $payment_method
 *   The payment method array.
 */
function commerce_chase_orbital_hpf_redirect_form_submit($order, $payment_method) {
  $orbital_transaction = drupal_json_decode($_GET['transaction']);
  // Prepare a transaction object to log the API response.
  $transaction = commerce_payment_transaction_new($payment_method['method_id'], $order->order_id);
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->amount = commerce_currency_decimal_to_amount($orbital_transaction['amount'], commerce_default_currency());
  $transaction->currency_code = commerce_default_currency();
  $transaction->payload['transaction'] = $orbital_transaction;
  $transaction->remote_id = $orbital_transaction['transId'];
  if ($payment_method['settings']['trans_type'] == 'auth_capture') {
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
  }
  else {
    $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
  }

  // Store the customers card on file token if it was returned.
  if (!empty($orbital_transaction['customerRefNum'])) {
    commerce_chase_orbital_hpf_create_cardonfile($order, $transaction, $orbital_transaction);
  }

  // Save the transaction information.
  commerce_payment_transaction_save($transaction);
}

/**
 * Save a commerce card on file entity from a Chase Orbital HPF response.
 *
 * @param $order
 *   The commerce order associated with the response.
 * @param $transaction
 *   The commerce payment transaction associated with the response.
 * @param $values
 *   The API response from Chase Orbital HPF.
 *
 * @return bool|object
 *   The commerce card on file object.
 */
function commerce_chase_orbital_hpf_create_cardonfile($order, $transaction, $values) {
  if ($order->uid > 0) {
    $cardonfile = commerce_cardonfile_new();
    $cardonfile->uid = $order->uid;
    $card_data = commerce_cardonfile_new();
    $card_data->uid = $order->uid;
    $card_data->payment_method = $transaction->payment_method;
    $card_data->instance_id = $transaction->instance_id;
    $card_data->remote_id = $values['customerRefNum'];
    $card_data->card_type = $values['ccType'];
    $card_data->card_name = $values['ccName'];
    $card_data->card_number = substr($values['ccNumber'], -4);
    $card_data->card_exp_month = $values['expMonth'];
    $card_data->card_exp_year = $values['expYear'];
    $card_data->status = 1;
    commerce_cardonfile_save($card_data);
    return $card_data;
  }
  return FALSE;
}
